//
// VMime library (http://www.vmime.org)
// Copyright (C) 2002 Vincent Richard <vincent@vmime.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Linking this library statically or dynamically with other modules is making
// a combined work based on this library.  Thus, the terms and conditions of
// the GNU General Public License cover the whole combination.
//

/* modified from defaultTimeoutHandler.hpp */

#ifndef VMIME_NET_PARAMETERIZABLETTIMEOUTHANDLER_HPP_INCLUDED
#define VMIME_NET_PARAMETERIZABLETTIMEOUTHANDLER_HPP_INCLUDED


#include "vmime/config.hpp"
#include "vmime/net/timeoutHandler.hpp"

#include <ctime>


/** A parameterized timeout handler for messaging services. The default action
  * is to throw a exceptions::operation_timed_out exception when an
  * operation is blocked for more than the specified number of seconds.
  */
class parameterized_timeout_handler : public vmime::net::timeoutHandler {

public:

	parameterized_timeout_handler(int timeout_seconds);
	~parameterized_timeout_handler();

	bool isTimeOut();
	void resetTimeOut();
	bool handleTimeOut();

private:

	time_t m_startTime;
	int timeout_seconds;
};

/** A class that creates parameterized timeout handlers.
  */
class parameterized_timeout_handler_factory :
	public vmime::net::timeoutHandlerFactory {

public:

	parameterized_timeout_handler_factory(int timeout_seconds);
	~parameterized_timeout_handler_factory();

	vmime::shared_ptr <vmime::net::timeoutHandler> create();

private:

	int timeout_seconds;
};


#endif // VMIME_NET_PARAMETERIZABLETTIMEOUTHANDLER_HPP_INCLUDED
